/*
 * ixpflash.h -- Header for Flash ROM Access Utilities on Intel(R) IXP1200 Evaluation Board
 *
 * Copyright (c) 1998, 1999 Cygnus Support
 *
 * The authors hereby grant permission to use, copy, modify, distribute,
 * and license this software and its documentation for any purpose, provided
 * that existing copyright notices are retained in all copies and that this
 * notice is included verbatim in any distributions. No written agreement,
 * license, or royalty fee is required for any of the authorized uses.
 * Modifications to this software may be copyrighted by their authors
 * and need not follow the licensing terms described here, provided that
 * the new terms are clearly indicated on the first page of each file where
 * they apply.
 *
 * Intel is a Registered Trademark of Intel Corporation.
 * ARM is a Registered Trademark of Advanced RISC Machines Limited.
 * Other Brands and Trademarks are the property of their respective owners.
 */

#ifndef __IXPFLASH_H__
#define __IXPFLASH_H__

#define min(a,b) ((a) < (b) ? (a):(b))
#define max(a,b) ((a) > (b) ? (a):(b))


#define PO2_8K         13  /* 2^13 = 8K */
#define PO2_16K        14  /* 2^14 = 16K */
#define PO2_128K       17

/* We use the mask to align an address at a block boundary. The block size
   is always a power of two. So depending on the block size choose an appropriate
   PO2_* from above and use it . Remember that if the block size is 8K use a block
   size value of 16K as we have two flash devices(16 bits each) interleaved. So the
   effective block size will be 16K and hence must use PO2_16K in the mask */


/*****************************************************************************
 Defines common for HOST and TARGET environment
*****************************************************************************/

#define IXPFLASH_DEFAULT_BASE_ADDR 0	//mmu off, flash ROM addr starts at 0
#define IXPFLASH_BLOCK_ALIGN_MASK  0xFFFF0000


/*******************************************************************************************************
 Define for Flash Commands
*******************************************************************************************************/

#define F28CMD_READ_ID        0x90909090
#define F28CMD_ERASE_SETUP    0x20202020
#define F28CMD_ERASE_CONFIRM  0xd0d0d0d0
#define F28CMD_PROGRAM_SETUP  0x40404040
#define F28CMD_READ_ARRAY     0xFFFFFFFF
#define F28CMD_READ_STATUS    0x70707070
#define F28CMD_CLEAR_STATUS   0x50505050
#define F28CMD_RESET          F28CMD_READ_ARRAY

#define F28_MANUF_ID          0x0089
#define FTYPE_28F800B         0x8893

#define BIT(i) (1 << (i))

/*******************************************************************************************************/

typedef enum __bool
{

  false,
  true

} bool;

/*
 * IXP1200 Cygmon related flash ROM parameters
 * since current cygmon image size is about 140KB, for consevatism
 * we will reserve 256KB (4 blocks x 64KB/block) for cygmon image, and use 
 * next 128KB (2 blocks x 64KB/block), cygmon related parameters will not
 * require such big space, but since the physical layout of flash ROM on
 * IXP1200 evaluation board (two 8 Mbits chip with first and second bytes 
 * in a double word in first chip, and third and fourth bytes in second chip),
 * we need at least two blocks. In order to overwrite other flash ROM area,
 * we will restrict our write and erase only to these two block.
 */
#endif  /* __IXPFLASH_H__  */

