/*
 * mmu_h.S -- Header for mmu on Intel(R) IXP1200 Evaluation Board
 *
 * Copyright (c) 1998, 1999 Cygnus Support
 *
 * The authors hereby grant permission to use, copy, modify, distribute,
 * and license this software and its documentation for any purpose, provided
 * that existing copyright notices are retained in all copies and that this
 * notice is included verbatim in any distributions. No written agreement,
 * license, or royalty fee is required for any of the authorized uses.
 * Modifications to this software may be copyrighted by their authors
 * and need not follow the licensing terms described here, provided that
 * the new terms are clearly indicated on the first page of each file where
 * they apply.
 *
 * Intel is a Registered Trademark of Intel Corporation.
 * ARM is a Registered Trademark of Advanced RISC Machines Limited.
 * Other Brands and Trademarks are the property of their respective owners.
 */

#include <bsp/platform.h>

#define			L1_TABLE_ENTRIES             0x1000  @ 16kB table (word entries) 

@ Allow different sized L2 Page tables, each L2 entry needs 256 words (1KB)
#define			L2_TABLE_ENTRIES             0x100   @  1kB table (word entries) 

#define			TTB_SIZE                     SZ_32K

@ Translation Table at the end of DRAM, occupy 32 Kbytes
#define			TTB_BASE                     (DRAMBase + DRAMSize - TTB_SIZE)

@ Access Permissions
@ Depending on the setup, 0 represents one of these two access permissions
#define			PT_NO_ACCESS                 (0 << 10)
#define			PT_SVC_R                     (0 << 10)
#define			PT_SVC_RW                    (1 << 10)
#define			PT_NO_USR_W                  (2 << 10)
#define			PT_ALL_ACCESS                (3 << 10)

#define			PT_C_BIT                     (1 << 3)
#define			PT_B_BIT                     (1 << 2)
#define			PT_CB_BITS                   (PT_C_BIT + PT_B_BIT)


@ Entry type
#define			PT_SECTION                   2
#define			PT_PAGE                      1
#define			PT_INVALID                   0

#define			PT_SMALL                     2
#define			PT_LARGE                     1


@ Cached, buffered access, section translation
@ AP=11, domain0, C=1, B=1
#define			CACHED_BUFFERED_ACCESS       (PT_ALL_ACCESS + PT_CB_BITS + PT_SECTION)

@ Cached, non-buffered access, section translation
@ AP=11, domain0, C=1, B=0
#define			CACHED_NOBUFFER_ACCESS       (PT_ALL_ACCESS + PT_C_BIT + PT_SECTION)

@ Non-cached, buffered access, section translation
@ AP=11, domain0, C=0, B=1
#define			NOCACHE_BUFFERED_ACCESS      (PT_ALL_ACCESS + PT_B_BIT + PT_SECTION)

@ Non-cached, non-buffered access, section translation
@ AP=11, domain0, C=0, B=0
#define			NOCACHE_NOBUFFER_ACCESS      (PT_ALL_ACCESS + PT_SECTION)

#define			DOMAIN_CLIENT_ACCESS         1
#define			DOMAIN_MANAGER_ACCESS        3

@ Definitions used in conditional assembly of Icache, Dcache and Write Buffer
@ options

/*  We changed IC_ON, DC_ON and WB_ON to get rid off
 *  multiple definition warning
 */
#define			IC_ON                0x1000
#define			IC_OFF               0x0

#define			DC_ON                0x4
#define			DC_OFF               0x0

#define			WB_ON                0x8
#define			WB_OFF               0x0


@ Bit definitions for the control register: 


#define			EnableMMU                    0x1
#define			EnableAlignFault             0x2
#define			EnableDcache                 0x4
#define			EnableWB                     0x8
#define			EnableBigEndian              0x80
#define			EnableMMU_S                  0x100     @ selects MMU access checks 
#define			EnableMMU_R                  0x200           @ selects MMU access checks 
#define			EnableIcache                 0x1000

