/*
 * mmumacros.S -- Header for mmu macros on Intel(R) IXP1200 Evaluation Board
 *
 * Copyright (c) 1998, 1999 Cygnus Support
 *
 * The authors hereby grant permission to use, copy, modify, distribute,
 * and license this software and its documentation for any purpose, provided
 * that existing copyright notices are retained in all copies and that this
 * notice is included verbatim in any distributions. No written agreement,
 * license, or royalty fee is required for any of the authorized uses.
 * Modifications to this software may be copyrighted by their authors
 * and need not follow the licensing terms described here, provided that
 * the new terms are clearly indicated on the first page of each file where
 * they apply.
 *
 * Intel is a Registered Trademark of Intel Corporation.
 * ARM is a Registered Trademark of Advanced RISC Machines Limited.
 * Other Brands and Trademarks are the property of their respective owners.
 */

#include  <bsp/mmu_h.S>


@Coprocessor read of ID register 
@
    .macro
    RDCP15_ID $reg_number
    MRC p15, 0, $reg_number, c0, c0 ,0
    .endm

@Coprocessor read of Control register 
@
    .macro
    RDCP15_Control $reg_number
    MRC p15, 0, $reg_number, c1, c0 ,0

    .endm

@Coprocessor write of Control register 
@
    .macro 
    WRCP15_Control $reg_number
    MCR p15, 0, $reg_number, c1, c0 ,0

    .endm

@Coprocessor read of Translation Table Base reg. 
@
    .macro
    RDCP15_TTBase $reg_number
    MRC p15, 0, $reg_number, c2, c0 ,0

    .endm

@Coprocessor write of Translation Table Base reg. 
@
    .macro 
    WRCP15_TTBase $reg_number
    MCR p15, 0, $reg_number , c2, c0 ,0

    .endm

@Coprocessor read of Domain Access Control reg. 
@
    .macro
    RDCP15_DAControl $reg_number
    MRC p15, 0, $reg_number, c3, c0 ,0

    .endm

@Coprocessor write of Domain Access Control reg. 
@
    .macro 
    WRCP15_DAControl $reg_number
    MCR p15, 0, $reg_number, c3, c0 ,0

    .endm

/*@Coprocessor read of Fault Status register 
*/
    .macro
    RDCP15_FaultStatus $reg_number
    MRC p15, 0, $reg_number, c5, c0 ,0

    .endm

/*@Coprocessor write of Fault Status register 
*/

.macro 
    WRCP15_FaultStatus $reg_number
    MCR p15, 0, $reg_number, c5, c0 ,0

    .endm

@Coprocessor read of Fault Address register 
@
    .macro
    RDCP15_FaultAddress $reg_number
    MRC p15, 0, $reg_number, c6, c0 ,0

    .endm

@Coprocessor write of Fault Address register 
@
    .macro 
    WRCP15_FaultAddress $reg_number
    MCR p15, 0, $reg_number, c6, c0 ,0

    .endm

@Coprocessor cache control 
@Flush ICache + DCache
@
    .macro 
    WRCP15_FlushIC_DC $reg_number
    MCR p15, 0, $reg_number, c7, c7 ,0

    .endm

@Coprocessor cache control 
@Flush ICache
@
    .macro 
    WRCP15_FlushIC $reg_number
    MCR p15, 0, $reg_number, c7, c5 ,0

    .endm

@Coprocessor cache control 
@Flush DCache
@
    .macro 
    WRCP15_FlushDC $reg_number
    MCR p15, 0, $reg_number, c7, c6 ,0

    .endm

@Coprocessor cache control 
@Flush DCache entry
@
    .macro 
    WRCP15_CacheFlushDentry $reg_number
    MCR p15, 0, $reg_number, c7, c6 ,1

    .endm

@Coprocessor cache control 
@Clean DCache entry
@
    .macro 
    WRCP15_CleanDCentry $reg_number
    MCR p15, 0, $reg_number, c7, c10 ,1

    .endm

@Coprocessor cache control 
@Clean + Flush DCache entry
@
    .macro 
    WRCP15_Clean_FlushDCentry $reg_number
    MCR p15, 0, $reg_number, c7, c14 ,1

    .endm

@Coprocessor Drain Write Buffer 
@
    .macro 
    WRCP15_DrainWriteBuffer $reg_number
    MCR p15, 0, $reg_number, c7, c10 ,4

    .endm


@Coprocessor TLB control 
@Flush ITB + DTB
@
    .macro 
    WRCP15_FlushITB_DTB $reg_number
    MCR p15, 0, $reg_number, c8, c7 ,0

    .endm

@Coprocessor TLB control 
@Flush ITB
@
    .macro 
    WRCP15_FlushITB $reg_number
    MCR p15, 0, $reg_number, c8, c5 ,0

    .endm

@Coprocessor TLB control 
@Flush DTB
@
    .macro 
    WRCP15_FlushDTB $reg_number
    MCR p15, 0, $reg_number, c8, c6 ,0

    .endm

@Coprocessor TLB control 
@Flush DTB entry

    .macro 
    WRCP15_FlushDTBentry $reg_number
    MCR p15, 0, $reg_number, c8, c6 ,1

    .endm

@Coprocessor test/clock/idle control 
@Enable Clock Switching
@
    .macro 
    WRCP15_EnableClockSW $reg_number
    MCR p15, 0, $reg_number, c15, c1 ,2

    .endm

@Coprocessor test/clock/idle control 
@Disable Clock Switching
@
    .macro 
    WRCP15_DisableClockSW $reg_number
    MCR p15, 0, $reg_number, c15, c2 ,2

    .endm

@Coprocessor test/clock/idle control 
@Disable nMCLK output
@
    .macro 
    WRCP15_DisablenMCLK $reg_number
    MCR p15, 0, $reg_number, c15, c4 ,2

    .endm

@Coprocessor test/clock/idle control 
@Wait for Interrupt
@
    .macro 
    WRCP15_WaitInt $reg_number
    MCR p15, 0, $reg_number, c15, c8 ,2

    .endm
