/*   porting from Linux */
/*
 * skbuff.h -- Header for PCI Configuration on Intel(R) IXP1200 Evaluation Board
 *
 * Copyright (c) 1998, 1999 Cygnus Support
 *
 * The authors hereby grant permission to use, copy, modify, distribute,
 * and license this software and its documentation for any purpose, provided
 * that existing copyright notices are retained in all copies and that this
 * notice is included verbatim in any distributions. No written agreement,
 * license, or royalty fee is required for any of the authorized uses.
 * Modifications to this software may be copyrighted by their authors
 * and need not follow the licensing terms described here, provided that
 * the new terms are clearly indicated on the first page of each file where
 * they apply.
 *
 * Intel is a Registered Trademark of Intel Corporation.
 * ARM is a Registered Trademark of Advanced RISC Machines Limited.
 * Other Brands and Trademarks are the property of their respective owners.
 */

#ifndef __SKBUFF_H__
#define __SKBUFF_H__
/*
 *  porting from Linux *
 */


#define HAVE_ALLOC_SKB		/* For the drivers to know */
#define HAVE_ALIGNABLE_SKB	/* Ditto 8)		   */
#define SLAB_SKB 		/* Slabified skbuffs 	   */

#define CHECKSUM_NONE 0
#define CHECKSUM_HW 1
#define CHECKSUM_UNNECESSARY 2

struct sk_buff_head {
	struct sk_buff	* next;
	struct sk_buff	* prev;
    /*__u32       qlen;  */     /* Must be same length as a pointer for using debugging */
    unsigned int qlen;       /* Must be same length as a pointer for using debugging */
};

struct sk_buff {
	struct sk_buff	* next;			/* Next buffer in list 				*/
	struct sk_buff	* prev;			/* Previous buffer in list 			*/
	struct sk_buff_head * list;		/* List we are on				*/
/* DC    struct sock *sk;   */         /* Socket we are owned by           */
/* DC    struct timeval  stamp; */         /* Time we arrived              */
/*DC     struct device   *dev;  */         /* Device we arrived on/are leaving by      */


/*     struct  dst_entry *dst; */

	char		cb[48];

	unsigned int 	len;			/* Length of actual data			*/
	unsigned int	csum;			/* Checksum 					*/
/*    volatile char   used; */          /* Data moved to user and not MSG_PEEK      */
	unsigned char	is_clone,		/* We are a clone				*/
			cloned, 		/* head may be cloned (check refcnt to be sure). */
  			pkt_type,		/* Packet class					*/
  			pkt_bridged,		/* Tracker for bridging 			*/
  			ip_summed;		/* Driver fed us an IP checksum			*/
	unsigned short	protocol;		/* Packet protocol from driver. 		*/
	unsigned short	security;		/* Security level of packet			*/
	unsigned int	truesize;		/* Buffer size 					*/

	unsigned char	*head;			/* Head of buffer 				*/
	unsigned char	*data;			/* Data head pointer				*/
	unsigned char	*tail;			/* Tail pointer					*/
	unsigned char 	*end;			/* End pointer					*/
};

/* DC temp extern struct sk_buff *	alloc_skb(unsigned int size, int priority); */
extern struct sk_buff *	alloc_skb(unsigned int size);  /* DC temp */
extern struct sk_buff *	dev_alloc_skb(unsigned int size); 
extern void		skb_reserve(struct sk_buff *skb, unsigned int len);
extern unsigned char *	skb_put(struct sk_buff *skb, unsigned int len);
extern unsigned char *	skb_push(struct sk_buff *skb, unsigned int len);
extern unsigned char *	skb_pull(struct sk_buff *skb, unsigned int len);
extern void dev_free_skb(struct sk_buff *skb);

#endif  /* __SKBUFF_H__  */
