/*
 * init_ixp1200eb.S -- Initialization for Intel(R) IXP1200 Evaluation Board
 *
 * Copyright (c) 2000, 2001 Cygnus Support
 *
 * The authors hereby grant permission to use, copy, modify, distribute,
 * and license this software and its documentation for any purpose, provided
 * that existing copyright notices are retained in all copies and that this
 * notice is included verbatim in any distributions. No written agreement,
 * license, or royalty fee is required for any of the authorized uses.
 * Modifications to this software may be copyrighted by their authors
 * and need not follow the licensing terms described here, provided that
 * the new terms are clearly indicated on the first page of each file where
 * they apply.
 *
 * Intel is a Registered Trademark of Intel Corporation.
 * ARM is a Registered Trademark of Advanced RISC Machines Limited.
 * Other Brands and Trademarks are the property of their respective owners.
 */
	.file "init_ixp1200eb.S"
        .title "Init Code for Intel(R) IXP1200 Evaluation Board"

#include <bsp/platform.h>
#include <bsp/ixp1200macros.S>
#include __BOARD_HEADER__


#define MMU_ENABLE_VALUE               (ARM_MMU_ENABLED                | \
			                ARM_ADDRESS_FAULT_DISABLED     | \
				        ARM_DATA_CACHE_ENABLED         | \
                                        ARM_WRITE_BUFFER_ENABLED       | \
                                        ARM_LITTLE_ENDIAN              | \
                                        ARM_ACCESS_CHECKS_NONE         | \
                                        ARM_INSTRUCTION_CACHE_ENABLED  | \
                                        ARM_VIRTUAL_IVR_BASE_00000000)
	
	.text

        .align     4

FUNC_START _bsp_init_stack

#if DO_LED
	WLED	6
#endif // DO_LED
	/*
	 * Disable ROM address alias.
	 * First, force pc to "normal" address, then
	 * perform a bus write to disable the alias.
	 */
	ldr	pc, =2f		/* load absolute address of 2f */
    2:
        str     r0, [r0, -r0]	/* disable the ROM alias */

#if DO_LED
	WLED	4
#endif // DO_LED
	
        mov     r8, lr		// lr was set to c_start by the caller

	ldr	sp, =(SRAM_BASE + SZ_512K)

	/* Init page table. Pass sdram size. */
	ldr	r0, [sp]
42:

#if CALL_BSP_MMU_INIT
//  calling _bsp_mmu_init causes flaky boot, even if you
//  immediately return from it; so we don't enable MMU
        LDR     r5, =DRAM_SIZE
	bl	SYM_NAME(_bsp_mmu_init)
	mov	sp, r0		// save memory size on the stack

#endif // CALL_BSP_MMU_INIT

#if DO_LED
	WLED 0x17
        ldr     r1, =0f
#endif // DO_LED

        /*
         * Now set the MMU Control Register to actually
         * enable the MMU, etc.
	 * No need to do anything special with pc as
	 * all memory is direct mapped.
         */
/*
        ldr     r0, =MMU_ENABLE_VALUE
        mcr     ARM_CACHE_COPROCESSOR_NUM,          \
                ARM_COPROCESSOR_OPCODE_DONT_CARE,   \
                r0,                                 \
                ARM_CONTROL_REGISTER,               \
                ARM_COPROCESSOR_RM_DONT_CARE,       \
                ARM_COPROCESSOR_OPCODE_DONT_CARE
*/

	nop
	nop
	nop

@       mov     pc, r1
0:
	/*
	 * Get RAM size from stack and pass as argument. The
	 * return address is set up to send us to a C function.
	 */
@	mov     r0, sp
	
@	mov	pc, r8
/* need to change following to make boot manager work */
	ldr	sp, =(SRAM_BASE+SZ_512K)

//	ldr	sp, =(SRAM_BASE+64000)

	WLED 	0x9e
	b	SYM_NAME(c_start)

#if DO_LED
	WLED 	0xa1
#endif // DO_LED
3:	b	3b
FUNC_END _bsp_init_stack

