/*
 * porting from Linux
 */
/*
 * skbuff.c -- Support for Intel(R) IXP1200 Evaluation Board
 *
 * Copyright (c) 1998, 1999 Cygnus Support
 *
 * The authors hereby grant permission to use, copy, modify, distribute,
 * and license this software and its documentation for any purpose, provided
 * that existing copyright notices are retained in all copies and that this
 * notice is included verbatim in any distributions. No written agreement,
 * license, or royalty fee is required for any of the authorized uses.
 * Modifications to this software may be copyrighted by their authors
 * and need not follow the licensing terms described here, provided that
 * the new terms are clearly indicated on the first page of each file where
 * they apply.
 *
 * Intel is a Registered Trademark of Intel Corporation.
 * ARM is a Registered Trademark of Advanced RISC Machines Limited.
 * Other Brands and Trademarks are the property of their respective owners.
 */

#include <string.h>
#include <bsp/bsp.h>
#include <bsp/cpu.h>
#include <bsp/platform.h>
#include "bsp_if.h"
#include "net.h"
#include "bsplog.h"
#include <bsp/skbuff.h>

void *malloc(int size);
void free(void *buff);

static char * __skb_pull(struct sk_buff *skb, unsigned int len);

/* DC struct sk_buff *alloc_skb(unsigned int size,int gfp_mask) */
struct sk_buff *alloc_skb(unsigned int size)
{
	struct sk_buff *skb;
        unsigned char *data;

/* DC 	if (in_interrupt() && (gfp_mask & __GFP_WAIT)) {
		static int count = 0;
		if (++count < 5) {
#ifdef DEBUG_SKB
            bsp_printf("alloc_skb called nonatomically from interrupt %p\n");
#endif // DEBUG_SKB
		}
		gfp_mask &= ~__GFP_WAIT;
	}
*/
	/* Get the HEAD */
/*	skb = malloc(sizeof(struct sk_buff)); */
	skb = (struct sk_buff *)(malloc(sizeof(struct sk_buff)));
	if (skb == NULL)
		return NULL;

	/* Get the DATA. Size must match skb_add_mtu(). */
	size = ((size + 15) & ~15);
	data = (unsigned char *)(malloc(size));
	if (data != NULL)
	{

	/* Note that this counter is useless now - you can just look in the
	 * skbuff_head entry in /proc/slabinfo. We keep it only for emergency
	 * cases.
	 */

            skb->truesize = size;


	    /* Load the data pointers. */
	    skb->head = data;
	    skb->data = data;
	    skb->tail = data;
	    skb->end = data + size;

	    /* Set up other state */
	    skb->len = 0;
	    skb->is_clone = 0;
	    skb->cloned = 0;

	    return skb;
        }
        else 
        {
             free(skb);
	     return NULL;
        }
}

struct sk_buff *dev_alloc_skb(unsigned int length)
{
	struct sk_buff *skb;

	skb = alloc_skb(length+16);
	if (skb)
		skb_reserve(skb,16);
	return skb;
}

unsigned char * skb_put(struct sk_buff *skb, unsigned int len)
{
	unsigned char *tmp=skb->tail;
	skb->tail+=len;
	skb->len+=len;
	if(skb->tail>skb->end)
	{
#ifdef DEBUG_SKB
        bsp_printf(" in *skb_put: skb->tail=%8.8X GT skb->end=%8.8X\n",
                   skb->tail, skb->end);
#endif // DEBUG_SKB
	}
	return tmp;
}

unsigned char * skb_push(struct sk_buff *skb, unsigned int len)
{
	skb->data-=len;
	skb->len+=len;
	if(skb->data<skb->head)
	{
#ifdef DEBUG_SKB
        bsp_printf(" in *skb_push: skb->data=%8.8X LT skb->head=%8.8X\n",
                   skb->data, skb->head);
#endif // DEBUG_SKB
	}
	return skb->data;
}

static char * __skb_pull(struct sk_buff *skb, unsigned int len)
{
	skb->len-=len;
	return 	skb->data+=len;
}

unsigned char * skb_pull(struct sk_buff *skb, unsigned int len)
{
	if (len > skb->len)
		return NULL;
	return __skb_pull(skb,len);
}

void skb_reserve(struct sk_buff *skb, unsigned int len)
{
	skb->data+=len;
	skb->tail+=len;
}

void dev_free_skb(struct sk_buff *skb)
{
    if(skb)
    {
        if(skb->head)
            free(skb->head);
        free(skb);
    }
}

