/*
 * system_timeri.c -- Driver for IXP1200 system timer
 *
 * Copyright (c) 1998, 1999 Cygnus Support
 *
 * The authors hereby grant permission to use, copy, modify, distribute,
 * and license this software and its documentation for any purpose, provided
 * that existing copyright notices are retained in all copies and that this
 * notice is included verbatim in any distributions. No written agreement,
 * license, or royalty fee is required for any of the authorized uses.
 * Modifications to this software may be copyrighted by their authors
 * and need not follow the licensing terms described here, provided that
 * the new terms are clearly indicated on the first page of each file where
 * they apply.
 */

#include <string.h>
#include <bsp/bsp.h>
#include <bsp/cpu.h>
#include <bsp/platform.h>
#include "bsp_if.h"
#include "bsplog.h"



/* local debug options */
/* #define SHOW_TX_INFO     1 */

/* #define SHOW_RX_INFO		1 */
/* #define USE_SW_GENERATED_INTERRUPT_FOR_TEST	1 */

#define BSP_IRQ_SYSTEM_TIMER	4

void ixp1200_sw_delay_msec(int msec);
unsigned long _bsp_ms_ticks(void); 
void _setup_system_timer_vec(); /* DC temp test */
void disable_hw_system_timer();
//DC since after enter cygmon monitor loop, hardware timer
//is no longer useable, start to use software emulated timer 
//unsigned int start_use_sw_timer=0;
unsigned int start_use_sw_timer=1;  //DC alway use software timer for now
static unsigned int system_time_in_ms=0;
static unsigned int sw_ms_ticks=0;

unsigned long get_ixp1200_system_time_in_ms(void);
static int system_timer_irq_handler(int irq_nr, void *regs);

unsigned long _bsp_ms_ticks(void)
{
    if(start_use_sw_timer)
    {
// We are artificially increment here
// because we do not have timers working

//DC modify bsp_ms_ticks() to increase 1 
//every time called, from measurement, in __arp_request() in arp.c 
//routine call MS_TICKS (finally call this routine) 13339 times
// about cost 1 sec, 10 retry cost about 10 sec,,
//should be tolerable 
        return (int)sw_ms_ticks += 1;
    } 
    else
        return get_ixp1200_system_time_in_ms();
}


void _setup_system_timer_vec() /* DC temp test */
{
 /* since IXP1200 is running at 162.202 Mhz,  then COUNTS_FOR_1MS is 162202 */
#define COUNTS_FOR_1MS 162202
    static bsp_vec_t system_timer_vec;
    /* static bsp_vec_t dummy_uart_vec; */

    *IXP1200_TIMER_1_LOAD=COUNTS_FOR_1MS;

    bsp_printf(" before setup_system_timer_vec *188=%8.8X, *IRQ=%8.8X  *PCI_IRQ=%8.8X CPSR=%8.8X \n", *IXP1200_PCI_IRQ_ENABLE_SET, *IXP1200_IRQ_STATUSX, *IXP1200_PCI_IRQ_STATUS, (__get_cpsr()) ); /* DC temp */

    system_timer_vec.handler = system_timer_irq_handler;
    system_timer_vec.next = NULL;

    bsp_install_vec(BSP_VEC_INTERRUPT, BSP_IRQ_SYSTEM_TIMER,
    BSP_VEC_REPLACE, &system_timer_vec);

    /* bsp_enable_irq(BSP_IRQ_SYSTEM_TIMER);  */
    *IXP1200_PCI_IRQ_ENABLE_SET |= IXP1200_PCI_IRQ_SYSTEM_TIMER_SET;
    /* bsp_printf("in _setup_system_timer_vec install system timer int_vec %d\n", BSP_IRQ_SYSTEM_TIMER); */  /* DC temp */
     __set_cpsr((__get_cpsr()) & 0xFFFFFF7F);  /* DC temp */
    bsp_printf(" after CPSR change, *IXP1200_PCI_IRQ_ENABLE_SET=%8.8X CPSR=%8.8X\n", *IXP1200_PCI_IRQ_ENABLE_SET, (__get_cpsr()) ); /* DC temp */
    bsp_printf(" after setup_system_timer_vec *188=%8.8X, *IRQ=%8.8X  *PCI_IRQ=%8.8X CPSR=%8.8X \n", *IXP1200_PCI_IRQ_ENABLE_SET, *IXP1200_IRQ_STATUSX, *IXP1200_PCI_IRQ_STATUS, (__get_cpsr()) ); /* DC temp */

     /* now start system timer */
     *IXP1200_TIMER_1_CONTROL=IXP1200_TIMER_CONTROL_ENABLE + IXP1200_TIMER_CONTROL_PERIODIC;
    bsp_printf(" after enable system timer *300=%8.8X, *308=%8.8X\n", *IXP1200_TIMER_1_LOAD, *IXP1200_TIMER_1_CONTROL);
}

static int system_timer_irq_handler(int irq_nr, void *regs)
{
    /* acknowledge interrupt: any write to Clear Register clears interrupt */
    *IXP1200_TIMER_1_CLEAR=0;
    system_time_in_ms++;
    bsp_printf(" TI ");
    if( (system_time_in_ms/60000) * 60000 == system_time_in_ms)
    {
         bsp_printf(" %d minutes\n", (system_time_in_ms/60000));
    }

    return 1;
}

unsigned long get_ixp1200_system_time_in_ms(void) 
{
    return (int)system_time_in_ms;
}

void disable_hw_system_timer()
{
}

void ixp1200_sw_delay_msec(int msec)
{
    int i=0, limit=(msec * 20)/11;
    unsigned char ch_space=0x20, ch_back_space=0x08;
    //since we can't use hardware timer, we will use this
    //emulated software delay to replace timer function,
    //from measurement, it take about 137 sec to executing
    //following statement 40000 times 
    limit=(msec * 50)/103;
    if(limit == 0) limit=1; 
    for (i=0; i < limit; i++)
        bsp_printf("%c%c", ch_space, ch_back_space);
}
