/* Kermit server, with S-record and ELF loading support
 *
 * Copyright (c) 1998, 1999 Cygnus Solutions
 *
 * The authors hereby grant permission to use, copy, modify, distribute,
 * and license this software and its documentation for any purpose, provided
 * that existing copyright notices are retained in all copies and that this
 * notice is included verbatim in any distributions. No written agreement,
 * license, or royalty fee is required for any of the authorized uses.
 * Modifications to this software may be copyrighted by their authors
 * and need not follow the licensing terms described here, provided that
 * the new terms are clearly indicated on the first page of each file where
 * they apply.
 */

#ifdef HAVE_BSP
#include <bsp/bsp.h>
#include "cpu_info.h"
#endif
#include "board.h"
#include "monitor.h"
#include "tservice.h"

#include <unistd.h>
#include <elfload.h>
#include "kermit.h"

#ifdef HAVE_BSP
#include "fmt_util.h"
#endif

#define DEBUG_KERMIT	0
#define DEBUG_LED	0

#ifdef XXX
#define putDebugChar xputchar
#endif // XXX

// int write_memory (mem_addr_t *dst, int size, int amt, char *src);

void write_led (unsigned char);

#ifdef TARGET_HAS_COFFLOADER
extern void readCoffFile(PFLOAD func);
extern int isCoffFile (char *magic, int magicLen);
#endif

#ifdef TARGET_HAS_OMFLOADER
extern void readOmfFile(PFLOAD func);
extern int isOmfFile (char *magic, int magicLen);
#endif

#ifndef MAKE_CYGMON_ADDR
#define MAKE_CYGMON_ADDR MAKE_STD_ADDR
#endif

/* Describes the currently-being-processed input packet. */
struct Packet
{
  /* The text of the packet. */
  unsigned char *buffer;
  /* What type it is. */
  int type;
  /* Its sequence number. */
  int seqNum;
  /* The number of bytes remaining in the buffer. */
  int bufLen;
  /* The offset within the file of the current byte. */
  int currFileOffset;
  /* The current repeat value. */
  int currRepeatCount;
  /* Total number of repeats for this character. */
  int finalRepeatCount;
  /* The state we're in:
     0 -- looking for repeat character
     		(transition to 1 or 4)
     1 -- looking for 8-bit quoting character
     		(transition to 2 or 5)
     2 -- got an 8-bit quoting character, looking for remainder of sequence
		(transition to 3 or 5)
     3 -- got a control quoting character, looking for final character
     		(transition to 5)
     4 -- got a repeat character, looking for repeat count
		(transition to 1)
     5 -- got final character, storing into memory according to repeat count
     		(transition to 5 or 0)
   */
  int state;
  /* The byte value. */
  unsigned char resCh;
};

static struct Packet dataPkt;
static int firstkRead = 0;
static unsigned char firstk[1024];

/* Standard defaults. */
static char prefix = '\001';
static char quoteChar = '#';
static int quote8bit = 0;
static int repeatChar = 0;
static int maxPacketLen = 94;
static int paddingChar = 0;
static int paddingCnt = 0;
static int eolChar = 0;
static int extendedMode = 0;

/* The last ACK packet we sent. */
static unsigned char lastPacket [100];
/* Its start. */
static unsigned char *lastPacketStart;
/* And its length. */
static int lastPacketLen;
static void initDataLoad (void);
static int doDataLoad (mem_addr_t *dest, char *dest2, int fileOffset, int len);
static void finishDataLoad (void);

#ifdef CLIENT
/* Add a quoted version of DATA to the packet being built in **PACKET. */
void
quote (unsigned char data, unsigned char **packet)
{
  if ((data > 127) && quote8bit)
    {
      *((*packet)++) = quote8bit;
      data -= 127;
    }
  if ((quote8bit && (data == quote8bit))
      || (quoteChar == data)
      || (repeatChar && (data == repeatChar))
      || ((data < 32) || (data == 127)))
    {
      if ((data < 32) || (data == 127))
	data ^= 64;
      *((*packet)++) = '#';
    }
  *(*(packet)++) = data;
}
#endif

/* Decode MEMLEN bytes of PACKET into DST. 

   When we return, MEMLEN will contain the number of bytes remaining
   to be read into DST, and DST will point to the next byte to be
   written.

   If a positive value is returned, we have processed all the bytes
   in this packet with bytes still remaining in MEMLEN to be written. */

static int
dequote (struct Packet *packet, int fileOffset, mem_addr_t *dst, int *memLen)
{
  int packetCh = 0;
  int havePacketData = 1;
  int notDone = 1;

  while (*memLen != 0 && havePacketData && notDone)
    {
      if (packet->state != 5)
	{
	  /* Need new data. */
	  if (packet->bufLen == 0)
	    {
	      havePacketData = 0;
	    }
	  else
	    {
	      packetCh = (*(packet->buffer++)) & 255;
	      packet->bufLen--;
	    }
	}

      if (havePacketData)
	{
	  switch (packet->state)
	    {
	    case 0:
	      packet->resCh = 0;
	      packet->finalRepeatCount = 1;
	      packet->currRepeatCount = 0;
	      if (repeatChar && packetCh == repeatChar)
		{
		  packet->state = 4;
		  break;
		}
	      /* FALLTHROUGH */
	    case 1:
	      if (quote8bit && packetCh == quote8bit)
		{
		  packet->resCh = 128;
		  packet->state = 2;
		  break;
		}
	      /* FALLTHROUGH */
	    case 2:
	      if (packetCh == quoteChar)
		{
		  packet->state = 3;
		  break;
		}
	      /* FALLTHROUGH */
	    case 3:
	      if (packetCh > 127)
		{
		  packet->resCh |= 128;
		  packetCh &= 127;
		}

	      if (packet->state == 3 && packetCh >= 63 && packetCh < 96)
		{
		  packet->resCh |= (packetCh ^ 64);
		}
	      else
		{
		  packet->resCh |= packetCh;
		}
	      packet->state = 5;
	      /* FALLTHROUGH */
	    case 5:
	      {
		int x;
		
		for (x = packet->currRepeatCount;
		     x < packet->finalRepeatCount;
		     x++)
		  {
		    if (*memLen == 0)
		      break;
		    if (packet->currFileOffset >= fileOffset && dst != NULL)
		      {
// write_led (dst->addr & 0xff);
// breakpoint();
			write_memory (dst, 1, 1, &packet->resCh);
			ADD_OFFSET (dst, dst, 1);
			(*memLen)--;
		      }
		    packet->currFileOffset++;
		  }
		if (x == packet->finalRepeatCount)
		  {
		    packet->state = 0;
		  }
		else
		  {
		    packet->currRepeatCount = x;
		    notDone = 0;
		  }
	      }
	      break;

	    case 4:
	      packet->finalRepeatCount = FROMCHAR (packetCh);
	      packet->state = 1;
	      break;
	    }
	}
    }
#if DEBUG_LED
      write_led (0x63);
#endif // DEBUG_LED
  if (*memLen != 0)
    {
      return 1;
    }
  else
    {
      return 0;
    }
}

#ifdef CLIENT
/* Add CH to the packet being built in PTR. It also tries to handle repeated
   data.

   If CH is -1, flush any remaining contents to the packet.  */

void
addCharToPacket (int ch, unsigned char **ptr)
{
  /* The # of repeated characters counted so far. */
  static int count = 1;
  /* The previous character seen. */
  static int prevCh = -1;

  if (repeatChar)
    {
      if ((ch != prevCh || count == 94) && count > 1 && repeatChar)
	{
	  if (count > 3)
	    {
	      *((*ptr)++) = repeatChar;
	      *((*ptr)++) = TOCHAR(count);
	      quote (prevCh, ptr);
	    }
	  else
	    {
	      int x;

	      for (x = 0; x < count; x++)
		{
		  quote (prevCh, ptr);
		}
	    }
	}
      if (ch == prevCh && ch != -1)
	count++;
      else
	{
	  count = 1;
	  prevCh = ch;
	}
    }
  else
    {
      if (ch != -1)
	quote (prevCh, ptr);
    }
}
#endif

/* Given an appropriately-quoted packet in PACKETDATA, append a header and
   any needed EOL characters. We return the total length of the packet;
   PACKETDATA will be modified to point to the start of the packet.

   PACKETDATA must have sufficient room before its start to hold the header,
   and sufficient room after the end to hold the EOL characters. 

   SEQNUM is the sequence number of the packet. TYPE is the type of packet
   (one of ACKPACK, NAKPAK, STTPAK, FILPAK, DATPAK, EOFPAK, or BRKPAK).
   PACKETLEN is the length of the encoded data in PACKETDATA.    */
static int
composePacket (int seqNum, int type, int packetLen, unsigned char **packetData)
{
  unsigned char *ptr;
  int checkSum = 0;
  unsigned char *packet = *packetData;
  unsigned char *packetEnd = packet + packetLen;

  /* Extended packet. */
  if (packetLen > 91)
    {
      int ch;
      packet -= 7;

      packet [1] = TOCHAR(0);
      /* Is it + 1? Not real clear from the doc. Appears we don't include the
         header checksum in the length. */
      packet [4] = TOCHAR ((packetLen + 1) / 95);
      packet [5] = TOCHAR ((packetLen + 1) % 95);
      ch = packet[1] + packet[2] + packet[3] + packet[4] + packet[5];
      packet [6] = CHECKSUM(ch);
    }
  else
    {
      packet -= 4;
      /* Add 3 to include the SEQ, TYPE and CHECK fields. */
      packet [1] = TOCHAR(packetLen + 3);
    }
  packet [0] = prefix;
  packet [2] = TOCHAR(seqNum);
  packet [3] = type;

  for (ptr = packet + 1; ptr < packetEnd; ptr++)
    {
      checkSum += (*ptr);
    }
  packetLen = (*packetData + packetLen) - packet;
  packet [packetLen++] = CHECKSUM(checkSum);
  *packetData = packet;
  return packetLen;
}

/* Handle the capability packet in PACKET, of length LEN. Modify the packet
   contents to match our response; we return the length of the resulting
   capability packet. */
static int
capPacket (unsigned char *packet, int len)
{
  if (len > 0)
    maxPacketLen = FROMCHAR (packet[0]);
  if (len > 1)
    packet[1] = TOCHAR(5);
  if (len > 2)
    paddingCnt = FROMCHAR (packet[2]);
  if (len > 3)
    paddingChar = packet[3] ^ 64;
  if (len > 4)
    eolChar = packet[4];
  if (len > 5)
    quoteChar = packet[5];
  if (len > 6)
    {
      if (packet[6] != 'Y')
	{
	  if (packet[6] == 'N')
	    {
	      quote8bit = 0;
	    }
	  else
	    {
	      if ((packet[6] > 32 && packet [6] < 63)
		  || (packet[6] > 95 && packet[6] < 127))
		{
		  quote8bit = packet[6];
		}
	      else
		{
		  quote8bit = 0;
		}
	    }
	}
      packet[6] = quote8bit ? quote8bit : ' ';
    }
  if (len > 7)
    packet[7] = '1';
  if (len > 8)
    {
      if ((packet[8] > 32 && packet [8] < 63)
	  || (packet[8] > 95 && packet [8] < 127))
	{
	  repeatChar = packet[8];
	}
      else
	{
	  repeatChar = 0;
	}
    }
  if (len > 9)
    {
      /* Check for extended packet support. */
      int capas = FROMCHAR (packet[9]);
      if (capas & 2)
	{
	  int which = 9;

	  /* Skip CAPAS bytes */
	  while (packet[which] & 1)
	    {
	      which++;
	    }
	  which++;
	  extendedMode = 1;
	  if ((which + 2) < len)
	    {
	      maxPacketLen = FROMCHAR (packet[which + 1]) * 95 + FROMCHAR (packet[which + 2]);
	    }
	  else
	    maxPacketLen = 500;
	  /* We don't "really" do windows, but they work (and do provide
	     a performance improvement).

	     "Doing windows right" would require buffering up good packets
	     after a bad one is received. We can assume, however, that
	     bad packets should be pretty infrequent. Since we decode
	     into memory directly, there's no real use for the packet
	     ring buffer otherwise. */
	  packet[9] = TOCHAR(2|4);
	  packet[which + 1] = TOCHAR (MAXPACKET / 95);
	  packet[which + 2] = TOCHAR (MAXPACKET % 95);
	  packet[which] = TOCHAR(8);
	}
      else
	{
	  packet[9] = TOCHAR(0);
	}
    }
  return len;
}

/* Add PACKETCHAR to the packet being decoded in DEST. SEQNUM is the sequence
   number of the packet being decoded. 

   A return value of -1 indicates that the packet is in error. 0
   indicates that we need more data; a positive value represents that
   a complete packet has been received, and the value is the type of
   the packet. */

static int
decodePacket (int packetChar, unsigned char **dest, int *seqNum)
{
  static int state = 0;
  static int len = 0;
  static int checksum = 0;
  static int packetType = 0;
  static int dupPacket = 0;
  static int currPacket = 0;

  if (packetChar == -1 || dest == NULL)
    {
      state = 0;
      currPacket = 0;
      return 0;
    }

  *seqNum = (currPacket % 64);
 
  if (state > 0)
    checksum += packetChar;

  switch (state)
    {
    case 0:
      if (packetChar != prefix)
	{
	  return -1;
	}
      dupPacket = 0;
      checksum = 0;
      packetType = 0;
      len = 0;
      state = 1;
      return 0;
    case 1:
      len = FROMCHAR (packetChar);
      state = 2;
      return 0;
    case 2:
      {
	int cpack = (currPacket % 64);
	int prevp = ((currPacket - 1) % 64);

	if (currPacket == 0)
	  {
	    prevp = -1;
	  }
	if (prevp == FROMCHAR (packetChar))
	  {
	    dupPacket = 1;
	  }
	else if (cpack != FROMCHAR (packetChar))
	  {
	    state = 0;
	    return -1;
	  }
	state = 3;
	return 0;
      }
    case 3:
      if (dupPacket)
	packetType = DUPPAK;
      else
	packetType = packetChar;
      if (len == 0)
	{
	  state = 4;
	}
      else
	{
	  state = 10;
	  len -= 2;
	}
      return 0;
    case 4:
      /* Extended packet. */
      len = FROMCHAR (packetChar) * 95;
      state = 5;
      return 0;
    case 5:
      len += FROMCHAR (packetChar);
      state = 6;
      return 0;
    case 6:
      if (CHECKSUM (checksum - packetChar) != packetChar)
	{
	  state = 0;
	  return -1;
	}
      else
	{
	  state = 10;
	  return 0;
	}
    case 10:
      {
	if (len == 1)
	  {
	    state = 0;
	    if (CHECKSUM (checksum - packetChar) != packetChar)
	      {
		return -1;
	      }
	    else 
	      {
		currPacket++;
		return packetType;
	      }
	  }
	else
	  {
	    *((*dest)++) = packetChar;
	    len--;
	  }
	return 0;
      }
    }
  return -1;
}

/* Write the packet in DATA, of length LENGTH, to the remote side. */
static void
writePacket (unsigned char *data, int len)
{
  int x;

#if DEBUG_LED
      write_led (0x70);
#endif // DEBUG_LED

  for (x = 0; x < len; x++)
    {
      xputchar(data[x]);
//      putDebugChar (data[x]);
    }
}

/* Send an acknowledgement packet to packet SEQNUM containing
   appropriately-quoted data in DATA of length LEN. */

static void
sendAck (unsigned char *data, int len, int seqNum)
{
  lastPacketStart = lastPacket;
  memcpy (lastPacketStart, data, len);
  lastPacketLen = composePacket (seqNum, ACKPAK, len, &lastPacketStart);
  writePacket (lastPacketStart, lastPacketLen);
}

/* Send a NAK packet referring to packet SEQNUM. */
void
sendNak (int seqNum)
{
  unsigned char nakbuf[90], *nakStart = nakbuf + 10;

  int nakLen = composePacket (seqNum, NAKPAK, 0, &nakStart);
  writePacket (nakStart, nakLen);
}

/* Resend the last ACK packet we sent. */
static void
resendLastPacket ()
{
  writePacket (lastPacketStart, lastPacketLen);
}

/* Read a packet from the remote side, and return its contents in DATA.
   PKT is filled in with values that describe the packet. The returned
   value is either -1, indicating that the user pressed ^C, 0, indicating
   that a fatal error was received from xgetchar(), or a positive value
   representing the packet type received. */

static int
getPacket (unsigned char *data, struct Packet *pkt)
{
  int ch;
  unsigned char *st = data;
  int foundInitial = 0;

int nCnt = 0;

#if DEBUG_LED
      write_led (0x64);
#endif // DEBUG_LED

  while ((ch = xgetchar ()) >= 0)
    {
#if DEBUG_LED
      if (nCnt++ == 0x0b)
      {
	write_led ((unsigned char) ch);
	breakpoint();
      }
#endif // DEBUG_LED
      if (ch == '\003')
	{
	  return -1;
	}

      if (!foundInitial && ch == prefix)
	{
	  foundInitial = 1;
	}
      if (foundInitial)
	{
	  int state = decodePacket (ch, &data, &pkt->seqNum);
#if DEBUG_LED
      write_led (0x66);
#endif // DEBUG_LED
	  if (state > 0)
	    {
	      foundInitial = 0;
	      pkt->buffer = st;
	      pkt->bufLen = data - st;
	      pkt->state = 0;
	      pkt->type = state;
	      return state;
	    }
	  /* Bad packet. */
	  if (state < 0)
	    {
	      sendNak (pkt->seqNum);
	      foundInitial = 0;
	      data = st;
	    }
	}
    }
#if DEBUG_LED
      write_led (0x67);
#endif // DEBUG_LED
  return 0;
}

static void
decodeReset ()
{
  decodePacket (-1, NULL, NULL);
}

static void 
initPkt (struct Packet *p)
{
  p->bufLen = 0;
  p->type = 0;
  p->buffer = 0;
  p->currFileOffset = 0;
  p->state = 0;
  decodeReset ();
}

/* Read MEMLEN bytes of the current Kermit file represented by PKT,
   starting from FILEOFFSET, into DST.

   If DST is NULL, the read data will be discarded.
   If MEMLEN is negative, the entire file is read. (This is normally used
   with DST == NULL).

   A negative value will be returned if there is an error; otherwise,
   the number of bytes read will be returned instead. */

static int
getKermitImage (struct Packet *pkt,
		int fileOffset,
		mem_addr_t *dst,
		int memLen)
{
  int notDone = 1;
  int origLen = memLen;
  static unsigned char data[MAXPACKET+10];
  
  if (pkt->state == 6)
    {
      return -1;
    }

  while (notDone)
    {
      if (pkt->type == DATPAK 
	  && (dequote (pkt, fileOffset, dst, &memLen) == 0))
	{
	  notDone = 0;
	}
      else
	{
	  if (getPacket (data, pkt) < 0)
	    {
	      pkt->state = 6;
	      return -1;
	    }
	  switch (pkt->type)
	    {
	    case DUPPAK:
	      resendLastPacket ();
	      break;
	    case STTPAK:
	      pkt->bufLen = capPacket (pkt->buffer, pkt->bufLen);
	      sendAck (pkt->buffer, pkt->bufLen, pkt->seqNum);

#if DEBUG_LED
      write_led (0x67);
#endif // DEBUG_LED
	      break;
	    case FILPAK:
	    case DATPAK:
	    case EOFPAK:
	      /* Note that the EOF packet does not indicate that the
		 connection will be terminated; it merely says that we hit
		 the end of the file. A BRK packet will be sent afterwards. */
	      sendAck ((unsigned char *)"", 0, pkt->seqNum);
#if DEBUG_LED
      write_led (0x68);
#endif // DEBUG_LED
	      break;
	    case BRKPAK:
	      sendAck ((unsigned char *)"", 0, pkt->seqNum);
	      notDone = 0;
	      pkt->state = 6;
	      break;
	    }
	}
    }
#if DEBUG_LED
      write_led (0x69);
#endif // DEBUG_LED
  return origLen - memLen;
}

int
readRawBinary (PFLOAD func, mem_addr_t *addr)
{
  if (addr == NULL)
    {
      xprintf ("Must specify an address when loading raw binary images\n");
      return -1;
    }
  func (addr, NULL, -1, -1);
  return 0;
}

int
kermit_load_cmd (cmdmode_t mode)
{
  extern char **argvect;

  if (mode == USAGE)
    {
      usage ("download [addr]");
      return 0;
    }
  if (mode == SHORT_HELP)
    {
      short_help ("Download a file to memory via kermit");
      return 0;
    }
  if (mode == LONG_HELP)
    {
      kermit_load_cmd (USAGE); 
      long_help ("\
This is used to download a  file to memory via Kermit. You can\n\
also specify an optional address to load the file at, ignoring whatever\n\
address is specified in the file.\n\
\n\
The file may contain either Motorola S-records, an ELF statically-linked\n\
executable, or a raw binary image.\n\
\n\
In the case of the raw binary image, an address must be specified on the\n\
command line.");
      return 0;
    }
  if (argvect[1] != NULL && argvect[2] != NULL)
    {
      return kermit_raw_cmd (USAGE);
    }
  else if (argvect[1] != NULL)
    {
      mem_addr_t addr;

      str2addr (argvect[1], &addr);

#if DEBUG_KERMIT
  xprintf ("loadFile(), Addr=%lx\n", addr.addr);
#endif // DEBUG_KERMIT

      loadFile (&addr);
      return 0;
    }
  else
    {
#if DEBUG_KERMIT
  xprintf ("loadFile() Addr=%lx\n", 0);
#endif // DEBUG_KERMIT
      loadFile (NULL);
      return 0;
    }
}

int
kermit_raw_cmd (cmdmode_t mode)
{
  extern char **argvect;

  if (mode == USAGE)
    {
      usage ("rawload addr");
      return 0;
    }
  if (mode == SHORT_HELP)
    {
      short_help ("Download a raw image to memory via kermit");
      return 0;
    }
  if (mode == LONG_HELP)
    {
      kermit_raw_cmd (USAGE); 
      long_help ("\
This is used to download a raw image file to memory via Kermit.");
      return 0;
    }
  if (argvect[1] == NULL || argvect[2] != NULL)
    {
      return kermit_raw_cmd (USAGE);
    }
  else
    {
      mem_addr_t addr;

      str2addr (argvect[1], &addr);

#if DEBUG_KERMIT
      xprintf ("kermit_raw_cmd() addr=%lx\n", addr.addr);
#endif

      readRawBinary (doDataLoad, &addr);
      finishDataLoad ();
      return 0;
    }
}

static void
initDataLoad (void)
{
  initPkt (&dataPkt);
  firstkRead = 0;
}

static int
doDataLoad (mem_addr_t *dest, char *dest2, int fileOffset, int len)
{
  mem_addr_t mem;
  int readSoFar = 0;

#if DEBUG_LED
  write_led (0x60);
#endif // DEBUG_LED

#if DEBUG_KERMIT
  xprintf ("doDataLoad() dest=%lx, dest2=%lx, fileOffs=%lx, len=%d\n",
	dest->addr, dest2, fileOffset, len);
#endif

  if (dest == NULL)
    {
      if (dest2 != NULL)
	{
	  dest = &mem;
	  MAKE_CYGMON_ADDR ((target_register_t)dest2, dest);
	}
    }
  else
    {
      mem = *dest;
      dest = &mem;
    }
  /* We allow the first 1024 bytes of the file to be perused multiple
     times. */
  if (firstkRead == 0)
    {
      mem_addr_t m2;

      MAKE_CYGMON_ADDR ((target_register_t)firstk, &m2);
      firstkRead = getKermitImage (&dataPkt, 0, &m2, 1024);

#if DEBUG_LED
      write_led (0x61);
#endif // DEBUG_LED

      if (firstkRead < 0)
	{
	  return -1;
	}
    }
  if ((fileOffset < firstkRead) && (len > 0))
    {
      readSoFar = len;
      /* Need to handle the case where the read starts in the first
	 1024 bytes but crosses the boundary. */
      if ((len + fileOffset) > firstkRead)
	{
	  readSoFar = firstkRead - fileOffset;
	}
      if (readSoFar > 0)
	{
// write_led (readSoFar);
// breakpoint();
	  write_memory (dest, 1, readSoFar, firstk + fileOffset);
	}

#if DEBUG_LED
      write_led (0x62);
#endif // DEBUG_LED

      if (len > readSoFar)
	{
	  fileOffset = firstkRead;
	  ADD_OFFSET (dest, dest, readSoFar);
	}
    }
  if (len > readSoFar)
    {
      return getKermitImage (&dataPkt, fileOffset, dest, len - readSoFar) 
	+ readSoFar;
    }
  return readSoFar;
}

static void
finishDataLoad (void)
{
  getKermitImage (&dataPkt, -1, NULL, -1);
}

int
isSRecordFile (char *magic, int amt)
{
  if (magic[0] == 'S')
    {
      int x;

      for (x = 0; x < amt; x++)
	{
	  if (magic[x] == '\r' || magic[x] == '\n')
	    {
	      break;
	    }
	}
      if (x < amt)
	{
	  return 1;
	}
    }
  return 0;
}

void
readSRecordFile (PFLOAD func, mem_addr_t *optAddr)
{
  int reclen = 0;
  static char recBuf[100];
  int32 offset = 0;
  
  while (1)
    {
      int x, i;
      int amt = func (NULL, recBuf + reclen, offset, 100 - reclen);

      if (amt > 0)
	{
	  reclen += amt;
	  offset += amt;
	}
      if (reclen <= 0)
	{
	  break;
	}
      if (recBuf[0] == '\r'|| recBuf[0] == '\n')
	{
	  for (x = 0; x < reclen; x++)
	    {
	      if (recBuf[x] != '\r' && recBuf[x] != '\n')
		{
		  break;
		}
	    }
	}
      else
	{
	  for (x = 0; x < reclen; x++)
	    {
	      if (recBuf[x] == '\r' || recBuf[x] == '\n')
		{
		  break;
		}
	    }
	  if (x == 100)
	    {
	      xprintf ("S record too long");
	      return;
	    }
	  recBuf [x++] = 0;
	  if (load_one_srecord (recBuf) <= 0)
	    {
	      break;
	    }
	}
      for (i = x; i < reclen ; i++)
	{
	  recBuf [i - x] = recBuf [i];
	}
      reclen -= x;
    }
}

void
loadFile (mem_addr_t *optAddr)
{
  static char magic[100];
  int amt;

#if DEBUG_KERMIT
  xprintf ("loadFile() entry, Addr=%lx\n", optAddr->addr);
#endif // DEBUG_KERMIT

  initDataLoad ();
  if ((amt = doDataLoad (NULL, magic, 0, 100)) <= 0)
    {
      return;
    }

  if (magic[0] == 127 && magic[1] == 'E' && magic[2] == 'L'
      && magic[3] == 'F')
    {
write_led (0x80);
      readElfFile (doDataLoad);
    }
#ifdef TARGET_HAS_COFFLOADER
  else if (isCoffFile (magic, amt))
    {
write_led (0x81);
      readCoffFile (doDataLoad);
    }
#endif
#ifdef TARGET_HAS_OMFLOADER
  else if (isOmfFile (magic, amt))
    {
write_led (0x82);
      readOmfFile (doDataLoad);
    }
#endif  
  else if (isSRecordFile (magic, amt))
    {
write_led (0x83);
      readSRecordFile (doDataLoad, optAddr);
    }
  else
    {
write_led (0x84);
      readRawBinary (doDataLoad, optAddr);
    }
  finishDataLoad ();
}
