/* Kermit server, with S-record and ELF loading support
 *
 * Copyright (c) 1998 Cygnus Solutions
 *
 * The authors hereby grant permission to use, copy, modify, distribute,
 * and license this software and its documentation for any purpose, provided
 * that existing copyright notices are retained in all copies and that this
 * notice is included verbatim in any distributions. No written agreement,
 * license, or royalty fee is required for any of the authorized uses.
 * Modifications to this software may be copyrighted by their authors
 * and need not follow the licensing terms described here, provided that
 * the new terms are clearly indicated on the first page of each file where
 * they apply.
 */

#ifndef KERMIT_H
#define KERMIT_H
#include "board.h"
#include <monitor.h>
#include <monitor_cmd.h>

#ifndef ASM
/* Maximum packet size. */
#define MAXPACKET 2048
/* Standard Kermit definitions. */
#define TOCHAR(c) ((c) + 32)
#define FROMCHAR(c) ((c) - 32)
#define CHECKSUM2(v) TOCHAR(((v) + ((v) / 64)) & 63)
#define CHECKSUM(v) CHECKSUM2((v)&255)

#define ACKPAK 'Y'
#define NAKPAK 'N'
#define STTPAK 'S'
#define FILPAK 'F'
#define DATPAK 'D'
#define EOFPAK 'Z'
#define BRKPAK 'B'
/* This packet was already received; the new one should be ignored. */
#define DUPPAK 1

/* Command for loading a raw binary image into memory. */
extern int kermit_raw_cmd (cmdmode_t mode);

/* Load an ELF, Motorola S-record or raw binary image into memory. */
extern int kermit_load_cmd (cmdmode_t mode);

/* Load a Kermit file into memory; if ADDR is non-null, it is used as the
   address to load into. */
extern void loadFile (mem_addr_t *addr);
#endif /* ASM */
#endif
