/* Monitor shell and main routines for CygMON the Wonder Monitor
 *
 * Copyright (c) 1997, 1998, 1999 Cygnus Solutions
 *
 * The authors hereby grant permission to use, copy, modify, distribute,
 * and license this software and its documentation for any purpose, provided
 * that existing copyright notices are retained in all copies and that this
 * notice is included verbatim in any distributions. No written agreement,
 * license, or royalty fee is required for any of the authorized uses.
 * Modifications to this software may be copyrighted by their authors
 * and need not follow the licensing terms described here, provided that
 * the new terms are clearly indicated on the first page of each file where
 * they apply.
 */

/* Platform-independent code for cygmon */
/*#define HAVE_BSP
*/
#include <stdlib.h>
#include <unistd.h>
#ifdef HAVE_BSP
#include <bsp/bsp.h>
#include <bsp/cpu.h>
#include <bsp/hex-utils.h>
#endif
#include <monitor.h>
#ifdef HAVE_BSP
#include "cpu_info.h"
#endif
#include <ledit.h>
#include <signal.h>
#include <string.h>
#include <ctype.h>

#include "cygdebflags.h"

#define IXP1200EB_SRAM_BASE	0x10000000	
#define IXP1200EB_SRAM_SIZE	0x00200000	//2 MB
//#define  CYGMON_VECTORS_SA	(IXP1200EB_SRAM_BASE + IXP1200EB_SRAM_SIZE - 64)	
#define  CYGMON_VECTORS_SA	(IXP1200EB_SRAM_BASE + IXP1200EB_SRAM_SIZE - 20480)	
#define  CYGMON_ROM_SA		0x00140000
// #define DEBUG_EXCEPT
// #define DEBUG_DBG_HANDLER
extern int is_ixp1200_central_func;
extern unsigned int flash_is_valid;
extern unsigned int lip_bootp;
extern unsigned int rip_bootp;
extern int save_parameters_into_flash();
extern int bo_assign_method;
extern unsigned int bo_lip;
extern unsigned int bo_rip;
extern char bo_kernel_file[128];
extern char bo_ramdisk_file[128];
extern unsigned int bo_countdown; 
extern void ixp1200_sw_delay_msec(int msec);
extern int ixp1200_uart_poll_read(char *ch);
extern void invoke_golinux_cmd();
extern char linux_zImage_file_path[128];
extern char linux_initrd_file_path[128];

#if USE_CYGMON_PROTOTYPES
/* Use common prototypes */
/* Some of the composed board.h files compose these
   prototypes redundently, but if they dont,
   these are the common definitions */
#include "fmt_util.h"   /* Interface to string formatting utilities */
#include "tservice.h"   /* Interface to target specific services */
#include "generic-stub.h" /* from libstub */
#endif /* USE_CYGMON_PROTOTYPES */
//after entering cygmon prompt, hardware timer will
//not function properly (due to interrupt not handled),
//switch to use software emulated timer, also disable 
//hardware system timer

extern unsigned int ixp1200_chip_rev; 
static int  cygmon_handle_exception (int sigval);
static void monitor_take_control (void);

#if CYGMON_SYSTEM_SERVICES
extern int process_syscall (int syscall_num);
#endif

int stub_is_active = 0;
mem_addr_t last_pc;

#ifdef HAVE_BSP
static int mon_dbg_handler(int exc_nr, void *regs);
static int mon_kill_handler(int exc_nr, void *regs);

#define __is_breakpoint_function() (get_pc() == (target_register_t)bsp_breakinsn)

/* Global pointer to current set of saved registers. */
void *mon_saved_regs;

/* Original BSP debug vector replaced by monitor. */
static bsp_handler_t old_dbg_vec;

#else // HAVE_BSP false
static int handle_signal (int signal_val);

__PFI user_signal_handler = NULL;
#endif

extern load_sram_vectors();

void show_sram_vector()
{
//    unsigned char *int_vect_data = 0x101F0000;
    unsigned char *int_vect_data = CYGMON_VECTORS_SA;
    unsigned char *bootmgr_vect_data = 
         (IXP1200EB_SRAM_BASE + IXP1200EB_SRAM_SIZE - 64);

    bsp_printf(" int_vec at %8.8X is:\n", (unsigned long)int_vect_data);
    dump_buf(int_vect_data, 64);
    bsp_printf(" bootmgr_vec at %8.8X is:\n", (unsigned long)bootmgr_vect_data);
    dump_buf(bootmgr_vect_data, 64);
}



#ifdef MONITOR_CONTROL_INTERRUPTS
/* This is set if the user wants interrupts enabled in the monitor. */
static int monitor_interrupts_enabled;
#endif


#if NOMAIN
int monitor_main (int argc, char **argv) /* Suppress default main() junk */
#else
int main (int argc, char **argv)     
#endif
{
#ifdef HAVE_BSP
  int  cur_port;
  struct bsp_comm_info comm_info;
#else
  /* Set up exception handling traps */
  initialize_stub ();
#endif
  unsigned int countdown=0;
  int i=0;
  unsigned int need_boot_linux=0;
  char in_ch=0;
  
//DC sram_vectors in the last 64 bytes in SRAM sometimes be overwrtten,
//need to reload here to make cygmon work properly
 	load_sram_vectors();
//	show_sram_vector();

  initialize_mon ();

#ifdef HAVE_BSP
  /* get info on debug channel */
  cur_port = bsp_set_debug_comm(-1);
  bsp_sysinfo(BSP_INFO_COMM, cur_port, &comm_info);

  /*
   * If we're using a network interface, don't install
   * the cygmon vectors. Currently, we only support stub
   * mode over a network connection.
   */
  if (comm_info.kind != BSP_COMM_ENET)
    {
      xprintf("\n");
#if DEBUG_CYGMON
      xprintf("About to print version info.. press enter key to continue\n");
#endif
/*	show_sram_vector();
	xprintf("testing show_sram_vector\n");
*/
      version ();
      /* replace original BSP debug and kill handler with ours */
      old_dbg_vec = bsp_install_dbg_handler(mon_dbg_handler);
      (void)bsp_install_kill_handler(mon_kill_handler);
    }
  else
    {
      /* This forces the console to use the gdb channel. */
      bsp_set_console_comm(cur_port);
    }
#else // HAVE_BSP NOT

  xprintf("\n");
  version ();

  __process_exception_vec = cygmon_handle_exception;
  __process_exit_vec = monitor_take_control;
#if CYGMON_SYSTEM_SERVICES
  __process_syscall_vec = process_syscall;
#endif  
  __process_signal_vec = handle_signal;
  __init_vec = install_breakpoints;
  __cleanup_vec = clear_breakpoints;
#endif // HAVE_BSP

//check whether user want to do linux boot
  
  if(is_ixp1200_central_func && flash_is_valid)
  {
    if( (bo_assign_method && lip_bootp && rip_bootp) ||
        (!bo_assign_method && bo_lip && bo_rip) )
    {
      need_boot_linux=1;
      countdown=bo_countdown;
      xprintf("Press space bar to stop Linux auto-boot....\n");
      while(countdown)  
      {
        xprintf("\r%3d", countdown);
        for(i=0; i < 20; i++)  //each countdown see 1 sec 
                               //check keyboard every 50 msec
        {
          ixp1200_sw_delay_msec(50);
          ixp1200_uart_poll_read(&in_ch);
          //xprintf("i=%d,  ch=%2.2X, i=%d",i, in_ch, i);
          if(in_ch == 0x20)
          { 
            need_boot_linux=0;
            countdown=1;
            break;
          }
        }
        countdown--; 
      }
      if(need_boot_linux)
      {
        xprintf("\r%3d\n", countdown);
        strcpy(linux_zImage_file_path, "/tftpboot/");
        strcat(linux_zImage_file_path, bo_kernel_file);
        //bsp_printf("kf=%s\n", linux_zImage_file_path);
        strcpy(linux_initrd_file_path, "/tftpboot/");
        strcat(linux_initrd_file_path, bo_ramdisk_file);
        //bsp_printf("rf=%s\n", linux_initrd_file_path);
        invoke_golinux_cmd();
      }
    }
  } 

    while (1)
    {
      breakpoint ();
xprintf("Returned from breakpoint\n");
      if (switch_to_stub_flag)
        {
	  xprintf("Switching to stub\n");
	  switch_to_stub_flag = 0;
	}
    }

  /* never reached */
  exit (0);
}


/* Transfer control to gdb stub */
 
int 
transfer_to_stub ()
{
  /* Return back to the exception handler, but the exception handler
     should invoke the stub's exception handler instead of ours. */
#ifdef HAVE_BSP
  (void)bsp_install_dbg_handler(old_dbg_vec);
#else
  __switch_to_stub ();
#endif

  /* The stub is now active. */
  stub_is_active = 1;
  return -1;
}

void
clear_user_state (void)
{
#ifdef HAS_TIMER
  if (__timer_enabled ())
    __settimer (0, 0);
#endif

  clear_breakpoints ();

#ifdef HAVE_BSP
  bsp_singlestep_cleanup(mon_saved_regs);
#else
  user_signal_handler = NULL;
  __clear_single_step ();
#endif
}

static void
monitor_take_control (void)
{
  stub_is_active = 0;

#ifdef HAVE_BSP
  /* replace original BSP debug trap handler with ours */
  (void)bsp_install_dbg_handler(mon_dbg_handler);
#else
  clear_user_state ();
  __process_exception_vec = cygmon_handle_exception;
#endif
}


#ifdef MONITOR_CONTROL_INTERRUPTS
void
monitor_enable_interrupts (void)
{
/* This is temporarily disabled
 * for testing
 */
    monitor_interrupts_enabled = 1;
    enable_interrupts ();
}

void
monitor_disable_interrupts (void)
{
  monitor_interrupts_enabled = 0;
  disable_interrupts ();
}

int
monitor_interrupt_state (void)
{
  return monitor_interrupts_enabled;
}
#endif // MONITOR_CONTROL_INTERRUPTS


static int
cygmon_handle_exception (int sigval)
{
  target_register_t pc;
  int		ret_result;

#if DEBUG_EXCEPT
    bsp_printf("cygmon_handle_exception() entry, sigval = %d\n", sigval);
#endif // DEBUG_EXCEPT


#ifdef MONITOR_CONTROL_INTERRUPTS
  if (monitor_interrupts_enabled)
    {
      if (! __in_interrupt)
	enable_interrupts ();
    }
#endif

#ifdef TARGET_EXCEPTION_CODE
  TARGET_EXCEPTION_CODE
#endif

#ifndef HAVE_BSP
  if (sigval != SIGKILL)
    if (handle_signal (sigval) == 0)
      return 0;
#endif

#if DEBUG_EXCEPT
  bsp_printf("cygmon_handle_exception() after handle_signal()\n");
#endif // DEBUG_EXCEPT

  clear_user_state ();

  /* We may want to tweak the PC to point at the faulting instruction,
     for example. (breakpoints on x86). */
#ifdef TARGET_ADJUST_PC
  TARGET_ADJUST_PC
#endif

  pc = get_pc();
  MAKE_STD_ADDR (pc, &last_pc);

#if DEBUG_EXCEPT
  xprintf ("Check sigval == SIGTRAP, sigval=%d, SIGTRAP=%d\n",
	sigval, SIGTRAP);

  xprintf ("__is_breakpoint_function()=%d\n", 
  	__is_breakpoint_function());
#endif // DEBUG_EXCEPT

  if ((sigval == SIGTRAP) && __is_breakpoint_function())
    {
      /*
       * This is the initial breakpoint inserted by the BSP
       * Don't print anything for this as it is confusing
       */
    }
  else
    {
      if (sigval == SIGTRAP)
          xprintf ("Hit breakpoint");
      else
          xprintf ("Got signal %d", sigval);

#if DEBUG_EXCEPT
      xprintf (" at 0x%s\n", int2str (pc, 16, sizeof (target_register_t) * 2));
#endif // DEBUG_EXCEPT

#ifdef DISASSEMBLER
      xprintf ("Disassembling\n");

      if (!__is_breakpoint_function ())
	{
	  do_dis (&last_pc);
	  flush_dis ();
	}
#endif
    }

#if DEBUG_EXCEPT
  bsp_printf("cygmon_handle_exception() -> monitor_take_control()\n");
#endif // DEBUG_EXCEPT

  monitor_take_control ();

#if DEBUG_EXCEPT
  bsp_printf("cygmon_handle_exception() -> monitor_loop()\n");
#endif // DEBUG_EXCEPT

  ret_result = monitor_loop ();

  bsp_printf("cygmon_handle_exception() monitor_loop return = %d\n",
	ret_result);
#if DEBUG_EXCEPT
#endif // DEBUG_EXCEPT

  return (ret_result);
}

#ifndef HAVE_BSP
/* Returns 0 if the program should restart at the point at which the
   signal was received, -1 otherwise. */
static int
handle_signal (int signal)
{
  if (signal == SIGKILL)
    return -1;

  if (user_signal_handler != NULL)
    {
      int result = user_signal_handler (signal);
      switch (result)
	{
	  /* Don't ignore potential hardware signals. */
	case 3:
	  if (signal == SIGSEGV || signal == SIGBUS || signal == SIGFPE
	      || signal == SIGTRAP || signal == SIGILL)
	    return -1;

	case 0:
	  return 0;

	default:
	case 1:
	case 2:
	  return -1;
	}
    }
    return -1;
}
#endif


void
version (void)
{
#ifdef HAVE_BSP
  struct bsp_platform_info platform;
  struct bsp_mem_info      mem;
  int                      i;
  unsigned long            u, totmem, topmem;
#endif
  extern char *build_date;

  xprintf ("Cygmon, the Cygnus ROM monitor.\n");
  xprintf ("Cygmon was written by Robert Richardson and Bob Manson of Cygnus Solutions\n");
  xprintf ("Copyright(c) 1997, 1998, 1999 Cygnus Solutions\n\n");
  xprintf ("Version: %s\nThis image was built on %s\n\n",
	   VERSION, build_date);

#ifdef HAVE_BSP
  bsp_sysinfo(BSP_INFO_PLATFORM, &platform);

  totmem = topmem = 0;
  i = 0;
  while (bsp_sysinfo(BSP_INFO_MEMORY, i++, &mem) == 0)
    {
      if (mem.kind == BSP_MEM_RAM)
        {
          totmem += mem.nbytes;
          u = (unsigned long)mem.virt_start + mem.nbytes;
	  if (u > topmem)
	    topmem = u;
        }
    }

  //xprintf("CPU: %s\n", platform.cpu);
  if(ixp1200_chip_rev == 0)
      xprintf("CPU: IXP1200 StrongARM(R) Microprocessor rev. A3\n");
  else
      xprintf("CPU: IXP1200 StrongARM(R) Microprocessor rev. B0\n");
  xprintf("Board: %s\n", platform.board);
  if (*(platform.extra))
      xprintf("%s\n", platform.extra);
/* 
 * We disabled reporting of memory size because
 * it was displayed incorrectly
 */
#ifdef REPORT_MEMORY_SIZE
  xprintf("Total RAM: %d bytes\n", totmem);
  xprintf("Top of RAM: 0x%x\n", topmem);
#endif // REPORT_MEMORY_SIZE
#endif
}


#ifdef HAVE_BSP
static int
mon_kill_handler(int exc_nr, void *regs)
{
    monitor_take_control();
    return 1;
}


static int mon_dbg_handler(int exc_nr, void *regs)
{
    int sig;
    unsigned long cur_pc;

#if DEBUG_DBG_HANDLER
    xprintf("mon_dbg_handler() entry\n");
#endif

    mon_saved_regs = regs;

#if DEBUG_CYGMON
    xprintf(" ->bsp_get_signal() \n");
#endif

    sig = bsp_get_signal(exc_nr, regs);

#if DEBUG_DBG_HANDLER
    xprintf("mon_dbg_handler, sig = %d\n", sig);
#endif // DEBUG_DBG_HANDLER

#if DEBUG_DBG_HANDLER
    xprintf("mon_dbg_handler() -> cygmon_handle_exception()\n");
#endif // DEBUG_DBG_HANDLER

    cygmon_handle_exception(sig);

#if DEBUG_DBG_HANDLER
    xprintf("mon_dbg_handler, after cygmon_handle_exception()\n");
#endif // DEBUG_DBG_HANDLER

    cur_pc = bsp_get_pc(regs);

#if DEBUG_DBG_HANDLER
    xprintf("mon_dbg_handler, curr_pc = %lx\n", cur_pc);
#endif // DEBUG_DBG_HANDLER

    if (cur_pc == (unsigned long)bsp_breakinsn)
	bsp_skip_instruction(regs);

    if (!stub_is_active)
	install_breakpoints();

    return 1;
}
#endif


