/*
 * Generic monitor support for SPARC processors
 *
 * Copyright (c) 1997 Cygnus Solutions
 *
 * The authors hereby grant permission to use, copy, modify, distribute,
 * and license this software and its documentation for any purpose, provided
 * that existing copyright notices are retained in all copies and that this
 * notice is included verbatim in any distributions. No written agreement,
 * license, or royalty fee is required for any of the authorized uses.
 * Modifications to this software may be copyrighted by their authors
 * and need not follow the licensing terms described here, provided that
 * the new terms are clearly indicated on the first page of each file where
 * they apply.
 */

#include <string.h>
#include <monitor.h>

/* Mapping of register names to numbers */
struct regstruct regtab[] =
{
  {"regname1",  REGNUM1},  {"regname2",  REGNUM2},  
  /* ... */
  {0, 0}
};

int
read_memory (mem_addr_t *src, int size, int amt, char *dst)
{
  /* !!! Read memory. The definition given below is trivial, and
     may be correct for your machine.

     This calls into the stub. */
  return __read_mem_safe (dst, src->addr, size * amt);
}

int
write_memory (mem_addr_t *dst, int size, int amt, char *src)
{
  /* !!! Write memory. The definition given below is trivial, and
     may be correct for your machine. 

     This calls into the stub.
     */
  return __write_mem_safe (src, dst->addr, size * amt);
}

int
str2addr (char *string, mem_addr_t *res)
{
  /* !!! Convert a string into a mem_addr_t. */
  res->addr = str2int (string, 16);
  return 0;
}

void
addr2str (mem_addr_t *addr, char *dest)
{
  /* !!! Convert a mem_addr_t into a user-displayable string. */
  strcpy (dest, int2str (addr->addr, 16, sizeof (target_register_t) * 2));
}

void
set_breakpoint (struct bp *bp)
{
  /* !!! This may be correct for your processor, if it has instructions
     of constant length. */
  if (! bp->in_memory)
    {
      extern void breakinst ();

      /* Don't set a breakpoint at the current PC location. */
      if (bp->address.addr != get_register (PC))
	{
	  read_memory (&bp->address, 4, 1, (char *)(&bp->old_inst));
	  bp->in_memory = 1;
	  write_memory (&bp->address, 4, 1, (char *) breakinst);
	  flush_i_cache ();
	}
    }
}

void
clear_breakpoint (struct bp *bp)
{
  /* !!! This may be correct for your processor, if it has instructions
     of constant length. */
  if (bp->in_memory)
    {
      write_memory (&bp->address, 4, 1, (char *)(&bp->old_inst));
      bp->in_memory = 0;
      flush_i_cache ();
    }
}

#if 0 /* moved to libstub */

void
clear_single_step ()
{
  /* !!! You must supply a definition here. */
}

void
single_step ()
{
  /* !!! You must supply a definition here. */
}

#endif
